@@ -93,6 +93,7 @@ module Hedgehog.Internal.Gen (
   -- ** Choice
   , constant
   , element
+  , element_
   , choice
   , frequency
   , recursive
@@ -204,6 +205,7 @@ import qualified Data.Char as Char
 import           Data.Foldable (for_, toList)
 import           Data.Functor.Identity (Identity(..))
 import           Data.Int (Int8, Int16, Int32, Int64)
+import           Data.Kind (Type)
 import           Data.List.NonEmpty (NonEmpty)
 import qualified Data.List.NonEmpty as NonEmpty
 import           Data.Map (Map)
@@ -350,7 +352,7 @@ generalize =
 -- | Class of monads which can generate input data for tests.
 --
 class (Monad m, Monad (GenBase m)) => MonadGen m where
-  type GenBase m :: (* -> *)
+  type GenBase m :: (Type -> Type)
 
   -- | Extract a 'GenT' from a  'MonadGen'.
   --
@@ -799,23 +801,54 @@ golden x =
 --   > 2058
 --   > 2060
 --
-integral :: (MonadGen m, Integral a) => Range a -> m a
+integral :: forall m a. (MonadGen m, Integral a) => Range a -> m a
 integral range =
-  shrink (Shrink.towards $ Range.origin range) (integral_ range)
+  -- https://github.com/hedgehogqa/haskell-hedgehog/pull/413/files
+  let
+    origin_ =
+      Range.origin range
+
+    binarySearchTree bottom top =
+      Tree.Tree $
+        let
+          shrinks =
+            Shrink.towards bottom top
+          children =
+            zipWith binarySearchTree shrinks (drop 1 shrinks)
+        in
+          Tree.NodeT top children
+
+    createTree root =
+      if root == origin_ then
+        pure root
+      else
+        hoist Morph.generalize $
+          Tree.consChild origin_ $
+            binarySearchTree origin_ root
+
+  in
+    fromGenT . GenT $ \size seed ->
+      createTree $ integralHelper range size seed
 
 -- | Generates a random integral number in the [inclusive,inclusive] range.
 --
 --   /This generator does not shrink./
 --
 integral_ :: (MonadGen m, Integral a) => Range a -> m a
-integral_ range =
-  generate $ \size seed ->
-    let
-      (x, y) =
-        Range.bounds size range
-    in
-      fromInteger . fst $
-        Seed.nextInteger (toInteger x) (toInteger y) seed
+integral_ =
+  generate . integralHelper
+
+
+-- | Generates a random integral value from a range.
+integralHelper :: (Integral a, Num c) => Range a -> Size -> Seed -> c
+integralHelper range size seed =
+  let
+    (x, y) =
+      Range.bounds size range
+  in
+    fromInteger . fst $
+      Seed.nextInteger (toInteger x) (toInteger y) seed
+
 
 -- | Generates a random machine integer in the given @[inclusive,inclusive]@ range.
 --
@@ -1154,6 +1187,20 @@ element = \case
     n <- integral $ Range.constant 0 (length xs - 1)
     pure $ xs !! n
 
+-- | Randomly selects one of the elements in the list.
+--
+--   This generator does not shrink the choice of element.
+--
+--   /The input list must be non-empty./
+--
+element_ :: MonadGen m => [a] -> m a
+element_ = \case
+  [] ->
+    error "Hedgehog.Gen.element: used with empty list"
+  xs -> do
+    n <- integral_ $ Range.constant 0 (length xs - 1)
+    pure $ xs !! n
+
 -- | Randomly selects one of the generators in the list.
 --
 --   This generator shrinks towards the first generator in the list.
@@ -1190,10 +1237,13 @@ frequency = \case
           else
             pick (n - k) xs
 
+      iis =
+        scanl1 (+) (fmap fst xs0)
+
       total =
         sum (fmap fst xs0)
 
-    n <- integral $ Range.constant 1 total
+    n <- shrink (\n -> takeWhile (< n) iis) $ integral_ $ Range.constant 1 total
     pick n xs0
 
 -- | Modifies combinators which choose from a list of generators, like 'choice'
