@@ -77,7 +77,10 @@ module Hedgehog.Internal.Property (
   , evalM
   , evalIO
   , evalEither
+  , evalEitherM
   , evalExceptT
+  , evalMaybe
+  , evalMaybeM
 
   -- * Coverage
   , Coverage(..)
@@ -126,7 +129,7 @@ module Hedgehog.Internal.Property (
 
 import           Control.Applicative (Alternative(..))
 import           Control.DeepSeq (NFData, rnf)
-import           Control.Monad (MonadPlus(..))
+import           Control.Monad (MonadPlus(..), (<=<))
 import           Control.Monad.Base (MonadBase(..))
 import           Control.Monad.Catch (MonadThrow(..), MonadCatch(..))
 import           Control.Monad.Catch (SomeException(..), displayException)
@@ -754,10 +757,10 @@ assert b = do
 --   diff x (<=) 'r'
 -- @
 --
---   /This function behaves like the unix `diff` tool, which gives a `0` exit/
---   /code if the compared files are identical, or a `1` exit code code/
---   /otherwise. Like unix `diff`, if the arguments fail the comparison, a diff/
---   /is shown./
+--   This function behaves like the unix @diff@ tool, which gives a 0 exit
+--   code if the compared files are identical, or a 1 exit code code
+--   otherwise. Like unix @diff@, if the arguments fail the comparison, a
+--   /diff is shown.
 --
 diff :: (MonadTest m, Show a, Show b, HasCallStack) => a -> (a -> b -> Bool) -> b -> m ()
 diff x op y = do
@@ -831,6 +834,13 @@ evalEither = \case
   Right x ->
     pure x
 
+-- | Fails the test if the action throws an exception, or if the
+--   'Either' is 'Left', otherwise returns the value in the 'Right'.
+--
+evalEitherM :: (MonadTest m, Show x, MonadCatch m, HasCallStack) => m (Either x a) -> m a
+evalEitherM =
+  evalEither <=< evalM
+
 -- | Fails the test if the 'ExceptT' is 'Left', otherwise returns the value in
 --   the 'Right'.
 --
@@ -838,6 +848,23 @@ evalExceptT :: (MonadTest m, Show x, HasCallStack) => ExceptT x m a -> m a
 evalExceptT m =
   withFrozenCallStack evalEither =<< runExceptT m
 
+-- | Fails the test if the 'Maybe' is 'Nothing', otherwise returns the value in
+--   the 'Just'.
+--
+evalMaybe :: (MonadTest m, Show a, HasCallStack) => Maybe a -> m a
+evalMaybe = \case
+  Nothing ->
+    withFrozenCallStack $ failWith Nothing "the value was Nothing"
+  Just x ->
+    pure x
+
+-- | Fails the test if the action throws an exception, or if the
+--   'Maybe' is 'Nothing', otherwise returns the value in the 'Just'.
+--
+evalMaybeM :: (MonadTest m, Show a, MonadCatch m, HasCallStack) => m (Maybe a) -> m a
+evalMaybeM =
+  evalMaybe <=< evalM
+
 ------------------------------------------------------------------------
 -- PropertyT
 
