@@ -6,6 +6,7 @@
 {-# LANGUAGE DoAndIfThenElse #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TupleSections #-}
 
@@ -45,14 +46,14 @@ import qualified Data.List as List
 import           Data.Map (Map)
 import qualified Data.Map as Map
 import           Data.Maybe (mapMaybe, catMaybes)
-import           Data.Semigroup (Semigroup(..))
 import           Data.Traversable (for)
 
 import           Hedgehog.Internal.Config
 import           Hedgehog.Internal.Discovery (Pos(..), Position(..))
 import qualified Hedgehog.Internal.Discovery as Discovery
-import           Hedgehog.Internal.Property (Coverage(..), Label(..), LabelName(..))
+import           Hedgehog.Internal.Prelude
 import           Hedgehog.Internal.Property (CoverCount(..), CoverPercentage(..))
+import           Hedgehog.Internal.Property (Coverage(..), Label(..), LabelName(..))
 import           Hedgehog.Internal.Property (PropertyName(..), Log(..), Diff(..))
 import           Hedgehog.Internal.Property (ShrinkCount(..), PropertyCount(..))
 import           Hedgehog.Internal.Property (TestCount(..), DiscardCount(..))
@@ -608,25 +609,34 @@ ppTextLines =
 
 ppFailureReport :: MonadIO m => Maybe PropertyName -> TestCount -> FailureReport -> m [Doc Markup]
 ppFailureReport name tests (FailureReport size seed _ mcoverage inputs0 mlocation0 msg mdiff msgs0) = do
+  let
+    basic =
+      -- Move the failure message to the end section if we have
+      -- no source location or can't find the source file.
+      let
+        msgs1 =
+          msgs0 ++
+          (if null msg then [] else [msg])
+
+        docs =
+          concatMap ppTextLines msgs1 ++
+          maybe [] ppDiff mdiff
+      in
+        (docs, Nothing)
+
   (msgs1, mlocation) <-
     case mlocation0 of
       Nothing ->
-        -- Move the failure message to the end section if we have
-        -- no source location.
-        let
-          msgs1 =
-            msgs0 ++
-            (if null msg then [] else [msg])
+        return basic
 
-          docs =
-            concatMap ppTextLines msgs1 ++
-            maybe [] ppDiff mdiff
-        in
-          pure (docs, Nothing)
-
-      Just location0 ->
-        fmap (concatMap ppTextLines msgs0,) $
+      Just location0 -> do
+        mAdvanced <-
           ppFailureLocation (fmap WL.text $ List.lines msg) mdiff location0
+        case mAdvanced of
+          Just advanced ->
+            return (concatMap ppTextLines msgs0, Just advanced)
+          Nothing ->
+            return basic
 
   coverageLocations <-
     case mcoverage of
@@ -678,7 +688,7 @@ ppFailureReport name tests (FailureReport size seed _ mcoverage inputs0 mlocatio
     WL.punctuate WL.line .
     fmap (WL.vsep . fmap (WL.indent 2)) .
     fmap (id :: [Doc Markup] -> [Doc Markup]) .
-    filter (not . null) $
+    List.filter (not . null) $
     concat [
       with args $
         WL.punctuate WL.line
